
#define sclk 13
#define mosi 11
#define cs   10
#define dc   9
#define rst  8  

#include <Adafruit_GFX.h>    
#include <Adafruit_ST7735.h> 
#include <ArgentRadioShield.h>
#include <math.h>
#include <SPI.h>

//Adafruit_ST7735 tft = Adafruit_ST7735(cs, dc, mosi, sclk, rst);
Adafruit_ST7735 tft = Adafruit_ST7735(cs, dc, rst);

// APRS Buffers
#define BUFLEN (260)
char packet[BUFLEN];
int buflen = 0;
int cnt(1);

ArgentRadioShield argentRadioShield = ArgentRadioShield(&Serial);   

float latitude = 49.96;                   // reference LAT ( in degs. )
float longitude = 8.2;                    // reference LON ( in degs. )      
float wayPointLatitude, wayPointLongitude;
float latitudeRadians, wayPointLatitudeRadians, longitudeRadians, wayPointLongitudeRadians;
float distanceToWaypoint, bearing, deltaLatitudeRadians, deltaLongitudeRadians;
const float pi = 3.14159265;
const int radiusOfEarth = 6371; // in km



void setup()                     // Runs at startup
{
  Serial.begin(4800);            // RadioShield runs at 4800 baud
  delay(3);                      // Allow time for RadioShield setup
  Serial.println("B100");        // Set display contrast
  Serial.println("C");           // Clear screen
  delay(10);                     // Can take a bit to clear the LCD, so wait
  Serial.println("WWaiting");    // 'Waiting' message stays until we receive something

  tft.initR(INITR_BLACKTAB);   
  tft.setRotation(tft.getRotation() + 1); // rotate 90degs

  tft.fillScreen(ST7735_BLACK);
  tft.setTextSize(3);    
  tft.drawRoundRect(4,4,150,115,7,ST7735_BLUE);  
  tft.setCursor(38, 25);
  testdrawtext("APRS", ST7735_YELLOW);
  tft.setCursor(18, 55);
  testdrawtext("DECODER", ST7735_YELLOW);
  tft.setCursor(25, 100);
  tft.setTextSize(1);    
  testdrawtext("dj7oo@t-online.de", ST7735_WHITE);
  delay(4000);
  tft.fillScreen(ST7735_BLACK);
  tft.setTextSize(3);    
  tft.drawRoundRect(4,4,150,115,7,ST7735_YELLOW);  
  tft.setCursor(18, 23);
  testdrawtext("WAITING", ST7735_GREEN);
  tft.setCursor(53, 53);
  testdrawtext("FOR", ST7735_GREEN);
  tft.setCursor(46, 83);
  testdrawtext("DATA", ST7735_GREEN);
}

void loop()                      // Runs constantly after startup
{
  while (Serial.available()) {
    char ch = argentRadioShield.read();
    if (ch == '\n'){
      packet[buflen] = 0;
      show_packet();
      buflen = 0;
    } else if ((ch > 31 || ch == 0x1c || ch == 0x1d || ch == 0x27) && buflen < BUFLEN) {
      // Mic-E uses some non-printing characters
      packet[buflen++] = ch;
    }
  }
}

void show_packet() {
  char *call, *posit;
  char type;
  long lat, lon;
  double lat2, lon2;
  
//    Serial.println(packet);

if (argentRadioShield.decode_posit(packet, &call, &type, &posit, &lon, &lat)) 
//   {
//    Serial.print(call);
//    Serial.print(F(" "));
//    Serial.print(lat);
//    Serial.print(F(" ")); 
//    Serial.println(lon);
//    }

  wayPointLatitude = lat;   
  wayPointLongitude = lon; 

  wayPointLatitude = wayPointLatitude/1000000;   
  wayPointLongitude = wayPointLongitude/1000000; 
    
  distanceToWaypoint = calculateDistance();
  bearing = calculateBearing();    

if(distanceToWaypoint<1000) {
//    Serial.print(call);
//    Serial.print(F(" is ")); 
//    Serial.print(distanceToWaypoint,1);
//    Serial.print(F(" Km in "));
//    Serial.print(bearing,1);
//    Serial.println(F(" Degrees"));    
        
        lat2 = lat;
        lon2 = lon;
  
        String text1 = (call);
//    Serial.println(text1.length());
        int num = (text1.length());
 
        tft.fillScreen(ST7735_BLACK);
        tft.drawRoundRect(1,1,154,70,7,ST7735_WHITE);  
        if(num==9)  {         
        tft.setTextSize(2); 
        tft.setCursor(27, 10); }
        if(num==8)  {         
        tft.setTextSize(3);   
        tft.setCursor(7, 4); }
        if(num==7)  {         
        tft.setTextSize(3);   
        tft.setCursor(17, 4); }
        if(num==6)  {         
        tft.setTextSize(3);   
        tft.setCursor(27, 4); }
        if(num<=5)  {         
        tft.setTextSize(3);   
        tft.setCursor(37, 4); }      
        testdrawtext(call, ST7735_GREEN);          

        tft.setTextSize(2);    
        tft.setCursor(05, 30);
        testdrawtext("is ", ST7735_WHITE);     
        tft.print(distanceToWaypoint,1);
        testdrawtext(" Km", ST7735_WHITE);   // distance in Km   
//        testdrawtext(" Mi", ST7735_YELLOW);   // distance in Miles
        tft.setCursor(05, 50);
        testdrawtext("in ", ST7735_WHITE);     
        tft.print(bearing,1);
        testdrawtext(" Deg", ST7735_WHITE);   
        
        tft.setCursor(0, 75);
        testdrawtext("Lat:", ST7735_YELLOW);     
        tft.print(lat2/1000000,5);        
        tft.setCursor(0, 95);
        testdrawtext("Lon:", ST7735_YELLOW);     
        if(lon2/1000000<=-100) {
        tft.print(lon2/1000000,4); }
        else {
        if(lon < 10000000) {
        testdrawtext(" ", ST7735_YELLOW); } 
        tft.print(lon2/1000000,5);  }      

        tft.setTextSize(1);    
        tft.setTextColor(ST7735_WHITE);
        tft.setCursor(80, 115);
        if(cnt<100&&cnt>9) {
        tft.print("0") ; }
        else
        if(cnt<10) {
        tft.print("00") ; }      
        tft.println(cnt);
        cnt = cnt+1;
   }
}

//////////////////////////////////////////////////////////////////////////////////
void testdrawtext(char *text, uint16_t color) {
  tft.setTextColor(color);
  tft.setTextWrap(true);
  tft.print(text);
}

// convert degrees to radians
void radianConversion()
{
  deltaLatitudeRadians = (wayPointLatitude - latitude) * pi / 180;
  deltaLongitudeRadians = (wayPointLongitude - longitude)* pi / 180;
  latitudeRadians = latitude * pi / 180;
  wayPointLatitudeRadians = wayPointLatitude * pi / 180;
  longitudeRadians = longitude * pi / 180;
  wayPointLongitudeRadians = wayPointLongitude * pi / 180;
}

// calculate distance from present location to next way point
float calculateDistance()
{
  radianConversion();
  float a = sin(deltaLatitudeRadians/2) * sin(deltaLatitudeRadians/2) +
          sin(deltaLongitudeRadians/2) * sin(deltaLongitudeRadians/2) * 
          cos(latitudeRadians) * cos(wayPointLatitudeRadians);
  float c = 2 * atan2(sqrt(a), sqrt(1-a)); 
  float d = radiusOfEarth * c;     
    return d * 1;                  // distance in kilometers
//  return d * 0.621371192;        // distance in miles
}

// calculate bearing from present location to next way point
float calculateBearing()
{
  radianConversion();  
  float y = sin(deltaLongitudeRadians) * cos(wayPointLatitudeRadians);
  float x = cos(latitudeRadians) * sin(wayPointLatitudeRadians) -
        sin(latitudeRadians) * cos(wayPointLatitudeRadians) * cos(deltaLongitudeRadians);
 bearing = atan2(y, x) / pi * 180;
  if(bearing < 0)
  {
    bearing = 360 + bearing;
  }  
  return bearing;
}

